package com.css.simulation.resource.scene.mapper;

import api.common.pojo.param.scene.ScenePackageParam;
import api.common.pojo.po.scene.ScenePackagePO;
import api.common.pojo.vo.scene.ScenePackageNewVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.springframework.stereotype.Repository;

import java.util.List;


@Mapper
@Repository
public interface ScenePackageMapper {

    void saveScenePackage(ScenePackagePO params);

    void updateScenePackage(ScenePackagePO params);

    void deleteScenePackage(ScenePackagePO params);

    //查询详情
    ScenePackagePO queryScenePackage(ScenePackageParam params);

    List<ScenePackagePO> queryScenePackageList(ScenePackageParam params);

    //单独给查询一个测试包的列表用
    ScenePackageNewVO queryScenePackageBySublist(String packageId);

    //测试包名查唯一
    ScenePackageNewVO queryByPackageName(ScenePackagePO params);

    ScenePackageNewVO querySelfOnly(@Param("packageName") String packageName, @Param("createUserId") String createUserId);

    //测试包数量
    Integer queryPackageCount(ScenePackagePO params);

    //改变状态是否禁用
    void updateIsUnavailable(ScenePackagePO params);

    //改变状态是否禁用
    void updateIsUnavailable2(ScenePackagePO params);

    @Update("update scene_package\n" +
            "set is_unavailable = #{isUnavailable}\n" +
            "where package_id in (select root_id from scene_package_sublist where scene_natural_ids like concat('%', #{sceneId}, '%'))")
    void updateIsUnavailableByNaturalId(@Param("isUnavailable") String isUnavailable, @Param("sceneId") String sceneId);

    @Update("update scene_package\n" +
            "set is_unavailable = #{isUnavailable}\n" +
            "where package_id in (select root_id from scene_package_sublist where scene_statue_ids like concat('%', #{sceneId}, '%'))")
    void updateIsUnavailableByStandardsId(@Param("isUnavailable") String isUnavailable, @Param("sceneId") String sceneId);

    @Update("update scene_package\n" +
            "set is_unavailable = #{isUnavailable}\n" +
            "where package_id in (select root_id from scene_package_sublist where scene_traffic_ids like concat('%', #{sceneId}, '%'))")
    void updateIsUnavailableByAccidentId(@Param("isUnavailable") String isUnavailable, @Param("sceneId") String sceneId);


    @Select("select is_unavailable from scene_package where package_id = #{packageId}")
    String selectIsUnavailableByPackageId(@Param("packageId") String packageId);

    @Update("update scene_package\n" +
            "set scene_num = scene_num - 1\n" +
            "where package_id in\n" +
            "      (select root_id from scene_package_sublist where scene_natural_ids like concat('%', #{naturalId}, '%'))")
    void updateSceneNumAfterDeleteSceneByNaturalId(@Param("naturalId") String naturalId);

    @Update("update scene_package\n" +
            "set scene_num = scene_num - 1\n" +
            "where package_id in\n" +
            "      (select root_id from scene_package_sublist where scene_statue_ids like concat('%', #{sceneId}, '%'))")
    void updateSceneNumAfterDeleteSceneByStandardsId(@Param("sceneId") String sceneId);

    @Update("update scene_package\n" +
            "set scene_num = scene_num - 1\n" +
            "where package_id in\n" +
            "      (select root_id from scene_package_sublist where scene_traffic_ids like concat('%', #{sceneId}, '%'))")
    void updateSceneNumAfterDeleteSceneByAccidentId(@Param("sceneId") String sceneId);
}
